@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ApacheResponse.h 2756 2007-12-11 10:57:59Z svn $
+ * $Id: ApacheResponse.h 2859 2008-04-12 14:30:08Z svn $
  *****************************************************************************/
 
 #ifndef APACHE_RESPONSE_H
@@ -37,6 +37,7 @@
 #include "apr_network_io.h"
 
 #include "Uncopyable.h"
+#include "Macro.h"
 
 
 class ApacheResponseWriter;
@@ -222,6 +223,23 @@ public:
         ap_set_last_modified(r->r);
 #else
         ap_set_last_modified(r);
+#endif
+    };
+    static void set_expires(Handle *r, apr_time_t time)
+    {
+        char *expire_str;
+
+        APR_PALLOC(expire_str, char *, r->pool, APR_RFC822_DATE_LEN);
+        apr_rfc822_date(expire_str, time);
+
+        apr_table_set(r->headers_out, "Expires", expire_str);
+    };
+    static void set_etag(Handle *r)
+    {
+#if AP_SERVER_MAJORVERSION_NUMBER == 1
+        ap_set_etag(r->r);
+#else
+        ap_set_etag(r);
 #endif
     };
     static void set_content_type(Handle *r)
