@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemManager.cpp 2756 2007-12-11 10:57:59Z svn $
+ * $Id: UploadItemManager.cpp 2772 2008-01-24 16:25:23Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -38,6 +38,7 @@
 #include "UploadItemManager.h"
 #include "UploadItem.h"
 #include "UploadItemList.h"
+#include "UploadItemIterator.h"
 #include "ThumbnailList.h"
 #include "TemporaryPool.h"
 #include "ReadLocker.h"
@@ -48,7 +49,7 @@
 
 using namespace std;
 
-SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2756 2007-12-11 10:57:59Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemManager.cpp 2772 2008-01-24 16:25:23Z svn $");
 
 #ifdef DEBUG
 #include <iostream>
@@ -128,8 +129,8 @@ void UploadItemManager::sync_all_item(apr_pool_t *pool, apr_size_t sync_limit)
 UploadItem *UploadItemManager::get_item(apr_pool_t *pool, apr_size_t item_id)
 {
     UploadItem *uitem;
-    ReadLocker manager_locker(lock_);
 
+    ReadLocker manager_locker(lock_);
     REQUIRE_MANAGER_LOCKED();
 
     APR_PALLOC(uitem, UploadItem *, pool, sizeof(UploadItem));
@@ -163,6 +164,14 @@ apr_file_t *UploadItemManager::get_item_file(apr_pool_t *pool,
     return file;
 }
 
+void UploadItemManager::update_atime(apr_size_t item_id)
+{
+    WriteLocker manager_locker(lock_);
+    REQUIRE_MANAGER_LOCKED();
+
+    AS_UITEM(item_list_->get_by_id(item_id))->update_atime();
+}
+
 UploadItemManager *UploadItemManager::get_instance(apr_shm_t *shm,
                                                    apr_pool_t *pool,
                                                    const char *data_dir_path,
@@ -263,7 +272,7 @@ apr_size_t UploadItemManager::add_impl(apr_pool_t *pool,
     UploadItem *uitem;
     apr_time_t current_time;
 
-    remove_old_files(pool, file_size);
+    remove_files(pool, file_size);
 
     {
         WriteLocker manager_locker(lock_);
@@ -287,7 +296,7 @@ apr_size_t UploadItemManager::add_impl(apr_pool_t *pool,
     }
 
     // 2 回に分けてロックしているので，最悪の場合を考えると必要
-    remove_old_files(pool, 0);
+    remove_files(pool, 0);
 
     return item_id;
 }
@@ -306,6 +315,16 @@ void UploadItemManager::remove_impl(apr_pool_t *pool, UploadItem *uitem)
     *mtime_ = apr_time_now();
 }
 
+void UploadItemManager::remove_files(apr_pool_t *pool,
+                                     apr_uint64_t file_size)
+{
+#ifdef REMOVE_UNPOPULAR
+    remove_unpopular_files(pool, file_size);
+#else
+    remove_old_files(pool, file_size);
+#endif
+}
+
 void UploadItemManager::remove_old_files(apr_pool_t *pool,
                                          apr_uint64_t file_size)
 {
@@ -325,6 +344,27 @@ void UploadItemManager::remove_old_files(apr_pool_t *pool,
     }
 }
 
+void UploadItemManager::remove_unpopular_files(apr_pool_t *pool,
+                                               apr_uint64_t file_size)
+{
+    // アクセスが無いファイルを削除
+    while (item_list_->is_full(file_size)) {
+        UploadItem *old_uitem;
+
+        UploadItemIterator item_iter(pool, this, 0, item_list_->size(),
+                                     UploadItem::cmp_file_atime);
+
+        WriteLocker manager_locker(lock_);
+        REQUIRE_MANAGER_LOCKED();
+
+        old_uitem = item_iter.get();
+
+        item_writer_.remove(pool, old_uitem);
+        thumbnail_list_->remove(old_uitem->get_id());
+        item_list_->remove(old_uitem->get_id());
+    }
+}
+
 void UploadItemManager::create_item_id_index(apr_size_t *item_id,
                                              apr_size_t *item_index) const
 {
