@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemReader.cpp 2756 2007-12-11 10:57:59Z svn $
+ * $Id: UploadItemReader.cpp 2812 2008-02-23 17:08:39Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -39,7 +39,7 @@
 #include "Macro.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: UploadItemReader.cpp 2756 2007-12-11 10:57:59Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemReader.cpp 2812 2008-02-23 17:08:39Z svn $");
 
 
 /******************************************************************************
@@ -53,32 +53,49 @@ UploadItemReader::UploadItemReader(apr_pool_t *pool, const char *data_dir_path,
 
 }
 
-void UploadItemReader::read(apr_size_t item_id, UploadItem *bbs_thread)
+void UploadItemReader::read(apr_size_t item_id, UploadItem *uitem)
 {
     const char *uitem_file_path;
     apr_mmap_t *uitem_file_map;
+    apr_size_t version;
 
     TemporaryPool temp_pool(pool_);
     uitem_file_path = get_data_path(temp_pool.get(), item_id);
     File uitem_file(temp_pool.get(), uitem_file_path);
 
-    read_and_check(&uitem_file, &uitem_file_map);
+    version = read_and_check(&uitem_file, &uitem_file_map);
+    switch (version) {
+    case 3:
+        memcpy(uitem, uitem_file_map->mm, sizeof(UploadItem::header_t));
+        break;
+    case 2:
+        // フォーマット変換
+        UploadItem::header_t *header;
+        UploadItem::header_ver2x_t *header_ver2x;
 
-    memcpy(bbs_thread, uitem_file_map->mm, sizeof(UploadItem));
-}
+        header = AS_UITEM_H(uitem);
+        header_ver2x = AS_UITEM_H_VER2x(uitem_file_map->mm);
 
-void UploadItemReader::read(apr_size_t item_id, UploadItem **bbs_thread,
-                            apr_pool_t *pool, File& uitem_file) const
-{
-    const char *uitem_file_path;
-    apr_mmap_t *uitem_file_map;
-
-    uitem_file_path = get_data_path(pool, item_id);
-    uitem_file = File(pool, uitem_file_path);
-
-    read_and_check(&uitem_file, &uitem_file_map);
-
-    *bbs_thread = AS_UITEM(uitem_file_map->mm);
+        new(header) UploadItem::header_t;
+        header->id             = header_ver2x->id;
+        header->index          = header_ver2x->index;
+        header->download_count = header_ver2x->download_count;
+        header->flags          = header_ver2x->flags;
+        header->file_size      = header_ver2x->file_size;
+        header->mtime          = header_ver2x->mtime;
+        strncpy(header->date, header_ver2x->date, ITM_MAX_DATE_SIZE);
+        strncpy(header->ip_address, header_ver2x->ip_address, ITM_MAX_IP_ADDRESS_SIZE);
+        strncpy(header->file_name, header_ver2x->file_name, ITM_MAX_FILE_NAME_SIZE);
+        strncpy(header->file_mime, header_ver2x->file_mime, ITM_MAX_FILE_MIME_SIZE);
+        strncpy(header->file_ext, header_ver2x->file_ext, ITM_MAX_FILE_EXT_SIZE);
+        strncpy(header->file_digest, header_ver2x->file_digest, ITM_MAX_FILE_DIGEST_SIZE);
+        strncpy(header->remove_pass, header_ver2x->remove_pass, ITM_MAX_REMOVE_PASS_SIZE);
+        strncpy(header->download_pass, header_ver2x->download_pass, ITM_MAX_DOWNLOAD_PASS_SIZE);
+        strncpy(header->comment, header_ver2x->comment, ITM_MAX_COMMENT_SIZE);
+        break;
+    default:
+        THROW(MESSAGE_BUG_FOUND);
+    }
 }
 
 bool UploadItemReader::is_exist_item(apr_size_t item_id) const
@@ -122,33 +139,48 @@ UploadItem *UploadItemReader::read(apr_pool_t *pool, const char *dir_path,
 /******************************************************************************
  * private メソッド
  *****************************************************************************/
-inline void UploadItemReader::read_and_check(File *uitem_file,
-                                            apr_mmap_t **uitem_file_map) const
+inline apr_size_t UploadItemReader::read_and_check(File *uitem_file,
+                                             apr_mmap_t **uitem_file_map)
 {
     uitem_file->open(APR_READ|APR_BINARY);
     *uitem_file_map = uitem_file->mmap(0, 0, APR_MMAP_READ, true);
 
-    check_format(*uitem_file_map);
+    return check_format(*uitem_file_map);
 }
 
-inline void UploadItemReader::check_format(apr_mmap_t *uitem_file_map)
+inline apr_size_t UploadItemReader::check_format(apr_mmap_t *uitem_file_map)
 {
     UploadItem *uitem;
+    apr_size_t version;
 
     uitem = AS_UITEM(uitem_file_map->mm);
 
-    if (uitem_file_map->size < sizeof(UploadItem)) {
+    // バージョン情報までのデータがあることを確認
+    if (uitem_file_map->size < (ITM_MAX_IDENTIFIER_SIZE+ITM_MAX_VERSION_SIZE)) {
         THROW(MESSAGE_UPLOAD_ITEM_FORMAT_INVALID);
     }
-
     if (UNLIKELY(strncmp(uitem->get_identifier(), PACKAGE_NAME,
                          ITM_MAX_IDENTIFIER_SIZE) != 0)) {
         THROW(MESSAGE_UPLOAD_ITEM_FORMAT_INVALID);
     }
-    // メジャーバージョン番号のみ比較する
-    if (UNLIKELY(strncmp(uitem->get_version(), PACKAGE_VERSION, 1) > 0)) {
+
+    version = (uitem->get_version())[0] - '0';
+    switch (version) {
+    case 3:
+        if (uitem_file_map->size < sizeof(UploadItem::header_t)) {
+            THROW(MESSAGE_UPLOAD_ITEM_FORMAT_INVALID);
+        }
+        break;
+    case 2:
+        if (uitem_file_map->size < sizeof(UploadItem::header_ver2x_t)) {
+            THROW(MESSAGE_UPLOAD_ITEM_FORMAT_INVALID);
+        }
+        break;
+    default:
         THROW(MESSAGE_PROGRAM_TOO_OLD);
     }
+
+    return version;
 }
 
 // Local Variables:
