@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: uploader_handler.cpp 2772 2008-01-24 16:25:23Z svn $
+ * $Id: uploader_handler.cpp 2852 2008-04-11 15:56:37Z svn $
  *****************************************************************************/
 
 #ifndef TEMPLATE_INSTANTIATION
@@ -57,7 +57,7 @@
 #include "SourceInfo.h"
 
 #ifndef TEMPLATE_INSTANTIATION
-SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2772 2008-01-24 16:25:23Z svn $");
+SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2852 2008-04-11 15:56:37Z svn $");
 #endif
 
 #define STR_CMP_LITERAL(arg, literal)                            \
@@ -844,27 +844,46 @@ static int input_pass(typename Response::Handle *r, UploaderConfig *config,
 }
 
 template<class Response>
-static int download_set_header(typename Response::Handle *r, UploadItem *uitem)
+static int download_set_header(typename Response::Handle *r, UploadItem *uitem,
+                               apr_file_t *item_file)
 {
 #ifdef UPLOADER_TYPE_APACHE
-        Response::set_content_type(r);
+    Response::set_content_type(r);
 #else
-        Response::set_content_type(r, uitem->get_file_mime());
+    Response::set_content_type(r, uitem->get_file_mime());
+#endif
+
+#ifdef UPLOADER_TYPE_APACHE
+#if AP_SERVER_MAJORVERSION_NUMBER != 1
+    if (item_file != NULL) {
+        const char *item_path;
+
+        apr_file_name_get(&item_path, item_file);
+        r->filename = const_cast<char *>(item_path);
+        r->finfo.filetype = APR_REG;
+        r->finfo.size = uitem->get_file_size();
+#if 0
+        r->finfo.inode = uitem->get_id(); // ETag 生成用
+
+        Response::set_etag(r);
+#endif
+    }
+#endif
 #endif
 
 #ifndef DEBUG
-        Response::set_modified_time(r, uitem->get_mtime());
-        Response::set_last_modified(r, uitem->get_mtime());
+    Response::set_modified_time(r, uitem->get_mtime());
+    Response::set_last_modified(r, uitem->get_mtime());
 
-        int status;
-        if ((status = Response::is_meets_condition(r)) != OK) {
-            return status;
-        }
+    int status;
+    if ((status = Response::is_meets_condition(r)) != OK) {
+        return status;
+    }
 #endif
 
-        Response::set_accept_ranges(r);
+    Response::set_accept_ranges(r);
 
-        return OK;
+    return OK;
 }
 
 // TODO: ApacheResponse.h に移動させる
@@ -891,7 +910,7 @@ static int download_impl(typename Response::Handle *r, UploaderConfig *config,
 {
     int status;
 
-    status = download_set_header<Response>(r, uitem);
+    status = download_set_header<Response>(r, uitem, item_file);
     if (status != OK) {
         Response::header_end(r);
         return status;
@@ -902,6 +921,7 @@ static int download_impl(typename Response::Handle *r, UploaderConfig *config,
         return OK;
     }
 
+    Response::set_expires(r, apr_time_now() +  apr_time_from_sec(VIW_EXPIRES));
     Response::set_content_disposition(r, mode);
 
 #if defined(USE_GZIP) && defined(UPLOADER_TYPE_APACHE)
@@ -940,7 +960,7 @@ static int download_prep(typename Response::Handle *r, UploaderConfig *config,
     int status;
 
     if (Response::is_head_method(r)) {
-        status = download_set_header<Response>(r, uitem);
+        status = download_set_header<Response>(r, uitem, NULL);
 
         Response::header_end(r);
         if (status != OK) {
@@ -1057,17 +1077,6 @@ static int download(typename Response::Handle *r, UploaderConfig *config,
             ->get_item_file(r->pool, item_id,
                             Response::Writer::is_sendfile_enabled(r));
 
-#ifdef UPLOADER_TYPE_APACHE
-#if AP_SERVER_MAJORVERSION_NUMBER != 1
-        {
-            const char *item_path;
-            apr_file_name_get(&item_path, item_file);
-            r->filename = const_cast<char *>(item_path);
-            r->finfo.filetype = APR_REG;
-            r->finfo.size = uitem->get_file_size();
-        }
-#endif
-#endif
         config->get_item_manager()->update_atime(item_id);
 
         if (is_view_mode) {
