@@ -29,6 +29,7 @@ run ./util/win_build.pl
 
 run make -f GNUmakefile.dist dist-sf
 run make -f GNUmakefile.dist dist-sf-rpm
+run make -f GNUmakefile.dist dist-sf-srpm
 run make -f GNUmakefile.dist dist-sf-deb
 run make -f GNUmakefile.dist dist-sf-win
 
@@ -39,8 +40,8 @@ run cp src/mod_uploader-ap2.2.so ${WEB_BASE}/file
     run cd doc
     run make doc-update
     run make doc-api
-    run cp apache.htm apache-win.htm ${WEB_BASE}/file
-    run cp -r api ${WEB_BASE}/file
+    run cp apache.htm apache.*.htm apache-win.htm ${WEB_BASE}
+    run cp -r api ${WEB_BASE}
 )
 
 (
